#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H

#include <nodelet/nodelet.h>
#include <ros/ros.h>

#include "tools/KalmanFilter.h"
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <logging/MessageOStream.h>

#include "car/ussDataMsg.h"
#include "car/camDataMsg.h"

using namespace boost::numeric::ublas;
namespace car
{
class Environment : public nodelet::Nodelet
{
public:
    void onInit() override;

    Environment();

private:
    ros::NodeHandle nh;
    MessageOStream messageOStream;

    double dt;
    ros::Time lastTimestamp;
    KalmanFilter kalmanFilter;

    ros::Publisher environmentData;
    ros::Subscriber ussData;
    ros::Subscriber camData;

    void ussDataCallback(const ussDataMsg::ConstPtr & inMsg);
    void camDataCallback(const camDataMsg::ConstPtr & inMsg);

    matrix<double> buildDynamicModelMatrix(double *deltaT);
    std::string buildPath();
};
}
#endif
