//
// Created by philipp on 22.03.18.
//

#ifndef MODULES_LOGGING_H
#define MODULES_LOGGING_H

#include <nodelet/nodelet.h>
#include <ros/ros.h>

#include <boost/asio.hpp>

#include "car/camDataMsg.h"
#include "car/ccDataMsg.h"
#include "car/environmentDataMsg.h"
#include "car/laneDataMsg.h"
#include "car/logEnabledMsg.h"
#include "car/logStringMsg.h"
#include "car/rcEnabledMsg.h"
#include "car/stmDataMsg.h"
#include "car/ussDataMsg.h"

#include "NetworkingLib/ServiceServer.h"
#include "MessageLib/Server.h"

namespace car
{
class Logging : public nodelet::Nodelet
{
public:
    static constexpr std::size_t PORT{10207};
    static constexpr std::size_t MAX_BUFFER_SIZE{std::numeric_limits<std::size_t>::max()};

    void onInit() override;

private:
    ros::NodeHandle nh;

    ros::Subscriber camData;
    ros::Subscriber ccData;
    ros::Subscriber environmentData;
    ros::Subscriber laneData;
    ros::Subscriber logEnabled;
    ros::Subscriber logString;
    ros::Subscriber rcEnabled;
    ros::Subscriber stmData;
    ros::Subscriber ussData;

    networking::Networking net;
    message::Server::Ptr messageServer;

    void camDataCallback(const camDataMsg::ConstPtr & inMsg);

    void ccDataCallback(const ccDataMsg::ConstPtr & inMsg);

    void environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg);

    void laneDataCallback(const laneDataMsg::ConstPtr & inMsg);

    void logEnabledCallback(const logEnabledMsg::ConstPtr & inMsg);

    void logStringCallback(const logStringMsg::ConstPtr & inMsg);

    void rcEnabledCallback(const rcEnabledMsg::ConstPtr & inMsg);

    void stmDataCallback(const stmDataMsg::ConstPtr & inMsg);

    void ussDataCallback(const ussDataMsg::ConstPtr & inMsg);
};

}

#endif //MODULES_LOGGING_H
