#ifndef EGOMOTION_H
#define EGOMOTION_H

#include <nodelet/nodelet.h>
#include <ros/ros.h>

#include <functional>
#include <atomic>
#include <logging/MessageOStream.h>

#include "car/environmentDataMsg.h"
#include "car/stmDataMsg.h"

namespace car
{
class EgoMotion
{
public:
    using Callback = std::function<void()>;

    explicit EgoMotion(ros::NodeHandle nh);

    void setPlatoonControllerNotify(const Callback & platoonControllerNotify)
    { this->platoonControllerNotify = platoonControllerNotify; }

    void setCruiseControllerNotify(const Callback & cruiseControllerNotify)
    { this->cruiseControllerNotify = cruiseControllerNotify; }

    float getDistance()
    { return distance; }

    float getPrvSpeed()
    { return prvSpeed; }

    float getOwnSpeed()
    { return stmSpeed; }

private:
    MessageOStream messageOStream;

    Callback platoonControllerNotify;
    Callback cruiseControllerNotify;

    std::atomic<float> distance{0};
    std::atomic<float> stmSpeed{0};
    std::atomic<float> relSpeed{0};
    std::atomic<float> prvSpeed{0};

    ros::Subscriber environmentData;
    ros::Subscriber stmData;

    void environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg);

    void stmDataCallback(const stmDataMsg::ConstPtr & inMsg);

    void computeAndNotify();
};
}

#endif
