#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "environment/Environment.h"
#include "exceptions/Exceptions.h"

#include "car/environmentDataMsg.h"

PLUGINLIB_EXPORT_CLASS(car::Environment, nodelet::Nodelet);

namespace car
{

Environment::Environment()
    : dt()
    , lastTimestamp(ros::Time::now())
    , kalmanFilter(1, buildDynamicModelMatrix(&dt), &dt, buildPath())
    , messageOStream(nh, "Environment")
{}

void Environment::onInit()
{
    messageOStream.write("onInit", "START");

    environmentData = nh.advertise<environmentDataMsg>("environmentData", 1);
    ussData = nh.subscribe("ussData", 1, &Environment::ussDataCallback, this);
    camData = nh.subscribe("camData", 1, &Environment::camDataCallback, this);

    messageOStream.write("onInit", "END");
}

void Environment::ussDataCallback(const ussDataMsg::ConstPtr & inMsg)
{
    vector<double> newMesVector(1);
    newMesVector(0) = (double) inMsg->distance;
    dt = (double) (inMsg->timestamp - lastTimestamp).toSec();
    lastTimestamp = inMsg->timestamp;

    vector<double> newState = kalmanFilter.nextIteration(newMesVector);

    environmentDataMsg outMsg;
    outMsg.distance = newState(0);
    outMsg.relativeSpeed = newState(1);
    environmentData.publish(outMsg);
}

void Environment::camDataCallback(const camDataMsg::ConstPtr & inMsg)
{
}

std::string Environment::buildPath()
{
    std::string userHome = getenv("HOME");
    return userHome + "/CarConfig/kalman.config";
}

matrix<double> Environment::buildDynamicModelMatrix(double * deltaT)
{
    /* TAKE CARE ON THE DIMENSION */
    *deltaT = 1.;
    matrix<double> dynamicModel(2, 2);
    dynamicModel(0, 0) = 1.0,
    dynamicModel(0, 1) = *deltaT;
    dynamicModel(1, 0) = 0;
    dynamicModel(1, 1) = 1.0;
    return dynamicModel;
}
}
