#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "lanekeeping/Lanekeeping.h"

#include "car/camDataMsg.h"
#include "car/environmentDataMsg.h"

PLUGINLIB_EXPORT_CLASS(car::Lanekeeping, nodelet::Nodelet);

namespace car
{

Lanekeeping::Lanekeeping()
    : messageOStream(nh, "Lanekeeping")
{}

void Lanekeeping::onInit()
{
    messageOStream.write("onInit", "START");

    laneData = nh.advertise<laneDataMsg>("laneData", 1);
    camData = nh.subscribe("camData", 1, &Lanekeeping::camDataCallback, this);

    messageOStream.write("onInit", "END");
}

void Lanekeeping::camDataCallback(const camDataMsg::ConstPtr & inMsg)
{
    float x_offset = inMsg->xoffset;
    messageOStream << MessageOStream::key << "x_offset"
                   << MessageOStream::value << x_offset
                   << MessageOStream::flush;
    laneDataMsg outMsg;
    outMsg.angle = 10*x_offset;
    laneData.publish(outMsg);
}

}
