//
// Created by philipp on 22.03.18.
//

#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>
#include <logging/MessageOStream.h>
#include "logging/Logging.h"

PLUGINLIB_EXPORT_CLASS(car::Logging, nodelet::Nodelet);

namespace car
{

constexpr std::size_t Logging::PORT;
constexpr std::size_t Logging::MAX_BUFFER_SIZE;

void Logging::onInit()
{
    messageServer = message::Server::create(net, PORT, MAX_BUFFER_SIZE);
    messageServer->run();
    *messageServer << message::Message{"Logging", "onInit", "START"};

    camData = nh.subscribe("camData", 1, &Logging::camDataCallback, this);
    ccData = nh.subscribe("ccData", 1, &Logging::ccDataCallback, this);
    environmentData = nh.subscribe("environmentData", 1, &Logging::environmentDataCallback, this);
    laneData = nh.subscribe("laneData", 1, &Logging::laneDataCallback, this);
    logEnabled = nh.subscribe("logEnabled", 1, &Logging::logEnabledCallback, this);
    logString = nh.subscribe("logString", 1000, &Logging::logStringCallback, this);
    rcEnabled = nh.subscribe("rcEnabled", 1, &Logging::rcEnabledCallback, this);
    stmData = nh.subscribe("stmData", 1, &Logging::stmDataCallback, this);
    ussData = nh.subscribe("ussData", 1, &Logging::ussDataCallback, this);

    *messageServer << message::Message{"Logging", "onInit", "END"};
}

void Logging::ccDataCallback(const ccDataMsg::ConstPtr & inMsg)
{
    //*messageServer << message::Message{"ccData", "speed", std::to_string(inMsg->speed)};
}

void Logging::camDataCallback(const camDataMsg::ConstPtr & inMsg)
{

}

void Logging::environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg)
{
    //*messageServer << message::Message{"environmentData", "distance", std::to_string(inMsg->distance)};
}

void Logging::laneDataCallback(const laneDataMsg::ConstPtr & inMsg)
{

}

void Logging::logEnabledCallback(const logEnabledMsg::ConstPtr & inMsg)
{

}

void Logging::logStringCallback(const logStringMsg::ConstPtr & inMsg)
{
    *messageServer << message::Message{inMsg->module, inMsg->key, inMsg->value};
}

void Logging::rcEnabledCallback(const rcEnabledMsg::ConstPtr & inMsg)
{

}

void Logging::stmDataCallback(const stmDataMsg::ConstPtr & inMsg)
{
    //*messageServer << message::Message{"stmData", "angle", std::to_string(inMsg->angle)};
    //*messageServer << message::Message{"stmData", "speed", std::to_string(inMsg->speed)};
}

void Logging::ussDataCallback(const ussDataMsg::ConstPtr & inMsg)
{
    *messageServer << message::Message{"ussData", "distance", std::to_string(inMsg->distance)};
}

}