#include "mainNode/CruiseController.h"

#include "car/ccDataMsg.h"

namespace car
{

CruiseController::CruiseController(ros::NodeHandle & nh,
                                   platoonProtocol::VehicleFacade & c2c,
                                   EgoMotion & egoMotion,
                                   PlatoonController & platoonController)
    : messageOStream(nh, "CruiseController")
      , c2c(c2c)
      , egoMotion(egoMotion)
      , platoonController(platoonController)
{
    ccData = nh.advertise<ccDataMsg>("ccData", 1);
}

void CruiseController::run()
{
    switch (platoonController.getCurrState())
    {
        case PlatoonState::ACC:
        {
            run_ACC(platoonController.getDesSpeed());
            break;
        }
        case PlatoonState::CACC_LV:
        {
            auto config = platoonController.getPlatoonConfig().get();
            run_ACC(config.platoonSpeed);
            c2c.setPlatoonSpeed(speed);
            break;
        }
        case PlatoonState::CACC_FV:
        {
            run_CACC();
            break;
        }
    }
}

void CruiseController::run_ACC(float target_speed)
{
    float v_self = egoMotion.getOwnSpeed();
    float d = egoMotion.getDistance();
    float crash_dist = crash_time * (v_self+2);

    if (d < crash_dist)
    {
         //messageOStream.write("in crash_dist",d);
         speed = target_speed * (d - stop_dist) / crash_dist;
    }
    else
    {
         //messageOStream.write("not in crash_dist",d);
        speed = target_speed;
    }

    speed = std::min(std::max(speed, min_speed), max_speed);

    ccDataMsg outMsg;
    outMsg.speed = speed;
    ccData.publish(outMsg);
}

void CruiseController::run_CACC()
{
    auto config = platoonController.getPlatoonConfig().get();
    float PS = config.platoonSpeed;
    float IPD = config.innerPlatoonDistance;
    float vp = egoMotion.getPrvSpeed();
    float d = egoMotion.getDistance();

    if (IPD < 1)
    { IPD = 1; }
    if (vp < eps && vp > -eps)
    {
        if (d < IPD)
        {
            speed = 0;
            return;
        }
        speed = units_conv_fac * 0.5 * (d - IPD) / IPD; // THIS IS DEPENDEND ON THE UNITS OF speed AND d!!!
    }
    else
    {
        speed = pow(d / IPD, dist_pow) * pow(PS / vp, speed_pow) * vp;
    }

    speed = std::min(std::max(speed, min_speed), max_speed);

    ccDataMsg outMsg;
    outMsg.speed = speed;
    ccData.publish(outMsg);
}


}
