//
// Created by philipp on 05.02.18.
//

#include "../../include/mainNode/NotifiableThread.h"

NotifiableThread::NotifiableThread(const Callback & callback)
    : callback(callback)
{
    runner = std::thread{
        [this]
        { run(); }};
}

NotifiableThread::~NotifiableThread()
{
    running = false;
    notifiedCondition.notify_one();
    runner.join();
}

void NotifiableThread::notify()
{
    std::unique_lock<std::mutex> lock(mutex);

    if (!running)
        return;

    notified = true;
    notifiedCondition.notify_one();
}

void NotifiableThread::run()
{
    while (running)
    {
        {
            std::unique_lock<std::mutex> lock(mutex);
            while (running && !notified)
                notifiedCondition.wait(lock);

            if (!running)
                return;

            notified = false;
        }

        callback();
    }
}
