#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "mavLink/MavLink.h"

#include "car/stmDataMsg.h"

PLUGINLIB_EXPORT_CLASS(car::MavLink, nodelet::Nodelet);

namespace car
{

MavLink::MavLink()
    : messageOStream(nh, "MavLink")
{}

void MavLink::onInit()
{
    messageOStream.write("onInit", "START");

    stmData = nh.advertise<stmDataMsg>("stmData", 1);
    ccData = nh.subscribe("ccData", 1, &MavLink::ccDataCallback, this);
    laneData = nh.subscribe("laneData", 1, &MavLink::laneDataCallback, this);
    rcEnabled = nh.subscribe("rcEnabled", 1, &MavLink::rcEnabledCallback, this);

    veloxConnection = veloxProtocol::Connection::create(net);
    veloxConnection->open(
        "/dev/ttySAC0",
        [this]
        { onStmDataReceived(); },
        [this]
        { messageOStream.write("ERROR", "UART was closed"); });

    messageOStream.write("onInit", "END");
}

void MavLink::ccDataCallback(const ccDataMsg::ConstPtr & inMsg)
{
    veloxConnection->setSpeed(convertSpeedToStm(inMsg->speed));
}

void MavLink::laneDataCallback(const laneDataMsg::ConstPtr & inMsg)
{
    veloxConnection->setSteeringAngle(inMsg->angle);
}

void MavLink::rcEnabledCallback(const rcEnabledMsg::ConstPtr & inMsg)
{
}

void MavLink::onStmDataReceived()
{
    stmDataMsg msg;
    float speed = speedFilter.moveWindow(convertSpeedFromStm(veloxConnection->getMeasuredSpeed().get()));
    msg.speed = speed;
    
    msg.angle = veloxConnection->getMeasuredSteeringAngle().get();
    
    stmData.publish(msg);
}

float MavLink::convertSpeedToStm(float speed) const
{
    return speed / 100;
}

float MavLink::convertSpeedFromStm(float speed) const
{
    return speed * 100;
}


}
