#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>
#include <fstream>
#include <exceptions/Exceptions.h>

#include "ultrasonic/Ultrasonic.h"

#include "car/ussDataMsg.h"

PLUGINLIB_EXPORT_CLASS(car::Ultrasonic, nodelet::Nodelet
);

namespace car
{

Ultrasonic::Ultrasonic()
    : messageOStream(nh, "Ultrasonic")
      , streamMedianFilter(5)
{
    sensor = UltrasonicSensor::create(net, readDevId());
}

void Ultrasonic::onInit()
{
    messageOStream.write("onInit", "START");

    ussData = nh.advertise<ussDataMsg>("ussData", 1);
    sensor->start(
        [this](auto distance)
         {
             ussDataMsg msg;
             msg.distance = streamMedianFilter.moveWindow(distance);
             msg.timestamp = ros::Time::now();
             ussData.publish(msg);
         });

    messageOStream.write("onInit", "END");
}

/** This reads ~/.CarConfig/sensor.config */
int Ultrasonic::readDevId()
{
    // open config file
    std::string userHome = getenv("HOME");
    std::ifstream configFile;
    std::string filename = userHome + "/CarConfig/sensor.config";
    configFile.open(filename, std::ifstream::in);
    if (!configFile.is_open())
    { throw FileNotFound{filename}; }

    // desired parameters
    int devId;
    bool devIdFound{false};
    std::string contentLine;
    while (!configFile.eof())
    {
        std::getline(configFile, contentLine);

        // split this line
        std::istringstream iss(contentLine);
        std::vector<std::string> words{std::istream_iterator<std::string>{iss}, {}};

        if (words.size() > 1)
        { // this line contains parameter[at(0)] and value[at(1)]
            if (words.at(0) == "devId:")
            {
                std::stringstream ss;
                ss << std::hex << words.at(1);
                ss >> devId;
                devIdFound = true;
            }
        }
    }

    if (!devIdFound)
    { throw MissingParameters(); }

    configFile.close();

    return devId;
}

}
